
.ag-theme-quartz {
    /*
    --ag-foreground-color: rgb(126, 46, 132);
    --ag-background-color: rgb(249, 245, 227) !important;
    --ag-header-foreground-color: rgb(204, 245, 172);
    --ag-header-background-color: rgb(209, 64, 129);
    --ag-odd-row-background-color: rgb(0, 0, 0, 0.03);
    --ag-header-column-resize-handle-color: rgb(126, 46, 132);
    */

    --ag-row-height: 40px;
    --ag-header-height: 40px;
    --ag-font-size: 15px;
    --ag-font-family: monospace;
}
